@@ -1,4 +1,11 @@
 
+#' @export
+as.vector.gpuVector <- function(x, mode = "any"){
+    out <- x[]
+    return(out)
+}
+
+
 #' @rdname as.gpuVector-methods
 #' @aliases as.gpuVector,vector
 setMethod('as.gpuVector', 
@@ -74,11 +81,11 @@ setMethod("Arith", c(e1="numeric", e2="gpuVector"),
               op = .Generic[[1]]
               switch(op,
                      `+` = {
-                         e1 = gpuVector(rep(e1, length(e2)), type=typeof(e2))
+                         e1 = gpuVector(rep(e1, length(e2)), type=typeof(e2), ctx_id = e2@.context_index)
                          gpuVec_axpy(1, e1, e2)
                          },
                      `-` = {
-                         e1 = gpuVector(rep(e1, length(e2)), type=typeof(e2))
+                         e1 = gpuVector(rep(e1, length(e2)), type=typeof(e2), ctx_id = e2@.context_index)
                          gpuVec_axpy(-1, e2, e1)
                          },
                      `*` = gpuVecScalarMult(e2, e1),
@@ -101,11 +108,11 @@ setMethod("Arith", c(e1="gpuVector", e2="numeric"),
               op = .Generic[[1]]
               switch(op,
                      `+` = {
-                         e2 = gpuVector(rep(e2, length(e1)), type=typeof(e1))
+                         e2 = gpuVector(rep(e2, length(e1)), type=typeof(e1), ctx_id = e1@.context_index)
                          gpuVec_axpy(1, e1, e2)
                          },
                      `-` = {
-                         e2 = gpuVector(rep(e2, length(e1)), type=typeof(e1))
+                         e2 = gpuVector(rep(e2, length(e1)), type=typeof(e1), ctx_id = e1@.context_index)
                          gpuVec_axpy(-1, e2, e1)
                          },
                      `*` = gpuVecScalarMult(e1, e2),
@@ -233,18 +240,14 @@ valueClass = "vector"
 
 # setOldClass("length")
 
-#' @title Length of gpuVector
-#' @description Get the length of a gpuR vector object
-#' @param x A gpuVector/vclVector object
-#' @return A numeric value
 #' @rdname length-methods
 #' @export
 setMethod('length', signature(x = "gpuVector"),
           function(x) {
               switch(typeof(x),
-                     "integer" = return(cpp_igpuVec_size(x@address)),
-                     "float" = return(cpp_fgpuVec_size(x@address)),
-                     "double" = return(cpp_dgpuVec_size(x@address))
+                     "integer" = return(cpp_gpuVector_size(x@address, 4L)),
+                     "float" = return(cpp_gpuVector_size(x@address, 6L)),
+                     "double" = return(cpp_gpuVector_size(x@address, 8L))
               )
               
           }
