@@ -11,6 +11,7 @@
 #' @param ncol An integer specifying the number of columns
 #' @param type A character string specifying the type of vclMatrix.  Default
 #' is NULL where type is inherited from the source data type.
+#' @param ctx_id An integer specifying the object's context
 #' @param ... Additional method to pass to vclMatrix methods
 #' @return A vclMatrix object
 #' @docType methods
@@ -25,15 +26,14 @@ setGeneric("vclMatrix", function(data = NA, nrow=NA, ncol=NA, type=NULL, ...){
 #' @aliases vclMatrix,matrix
 setMethod('vclMatrix', 
           signature(data = 'matrix'),
-          function(data, type=NULL){
+          function(data, type=NULL, ctx_id=NULL){
               
               if (is.null(type)) type <- typeof(data)
-              device_flag <- ifelse(options("gpuR.default.device.type") == "gpu", 0, 1)
-              
+
               device <- currentDevice()
               
-              context_index <- currentContext()
-              device_index <- device$device_index
+              context_index <- ifelse(is.null(ctx_id), currentContext(), as.integer(ctx_id))
+              device_index <- as.integer(device$device_index)
               device_type <- device$device_type
               device_name <- switch(device_type,
                                     "gpu" = gpuInfo(device_idx = as.integer(device_index))$deviceName,
@@ -43,15 +43,10 @@ setMethod('vclMatrix',
               platform_index <- currentPlatform()$platform_index
               platform_name <- platformInfo(platform_index)$platformName
               
-              if(type == "double" & !deviceHasDouble(platform_index, device_index)){
-                  stop("Double precision not supported for current device. 
-                       Try setting 'type = 'float'' or change device if multiple available.")
-              }
-              
               data = switch(type,
                             integer = {
                                 new("ivclMatrix", 
-                                    address=cpp_sexp_mat_to_vclMatrix(data, 4L, device_flag),
+                                    address=cpp_sexp_mat_to_vclMatrix(data, 4L, context_index - 1),
                                     .context_index = context_index,
                                     .platform_index = platform_index,
                                     .platform = platform_name,
@@ -60,7 +55,7 @@ setMethod('vclMatrix',
                             },
                             float = {
                                 new("fvclMatrix", 
-                                    address=cpp_sexp_mat_to_vclMatrix(data, 6L, device_flag),
+                                    address=cpp_sexp_mat_to_vclMatrix(data, 6L, context_index - 1),
                                     .context_index = context_index,
                                     .platform_index = platform_index,
                                     .platform = platform_name,
@@ -68,8 +63,9 @@ setMethod('vclMatrix',
                                     .device = device_name)
                             },
                             double = {
+                                assert_has_double(platform_index, device_index)
                                 new("dvclMatrix",
-                                    address = cpp_sexp_mat_to_vclMatrix(data, 8L, device_flag),
+                                    address = cpp_sexp_mat_to_vclMatrix(data, 8L, context_index - 1),
                                     .context_index = context_index,
                                     .platform_index = platform_index,
                                     .platform = platform_name,
@@ -89,15 +85,15 @@ setMethod('vclMatrix',
 #' @aliases vclMatrix,missing
 setMethod('vclMatrix', 
           signature(data = 'missing'),
-          function(data, nrow=NA, ncol=NA, type=NULL){
+          function(data, nrow=NA, ncol=NA, type=NULL, ctx_id = NULL){
               
               if (is.null(type)) type <- getOption("gpuR.default.type")
-              device_flag <- ifelse(options("gpuR.default.device.type") == "gpu", 0, 1)
+              #device_flag <- ifelse(options("gpuR.default.device.type") == "gpu", 0, 1)
               
               device <- currentDevice()
-              
-              context_index <- currentContext()
-              device_index <- device$device_index
+
+              context_index <- ifelse(is.null(ctx_id), currentContext(), as.integer(ctx_id))
+              device_index <- as.integer(device$device_index)
               device_type <- device$device_type
               device_name <- switch(device_type,
                                     "gpu" = gpuInfo(device_idx = as.integer(device_index))$deviceName,
@@ -107,15 +103,10 @@ setMethod('vclMatrix',
               platform_index <- currentPlatform()$platform_index
               platform_name <- platformInfo(platform_index)$platformName
               
-              if(type == "double" & !deviceHasDouble(platform_index, device_index)){
-                  stop("Double precision not supported for current device. 
-                       Try setting 'type = 'float'' or change device if multiple available.")
-              }
-              
               data = switch(type,
                             integer = {
                                 new("ivclMatrix", 
-                                    address=cpp_zero_vclMatrix(nrow, ncol, 4L, device_flag),
+                                    address=cpp_zero_vclMatrix(nrow, ncol, 4L, context_index - 1),
                                     .context_index = context_index,
                                     .platform_index = platform_index,
                                     .platform = platform_name,
@@ -124,7 +115,7 @@ setMethod('vclMatrix',
                             },
                             float = {
                                 new("fvclMatrix", 
-                                    address=cpp_zero_vclMatrix(nrow, ncol, 6L, device_flag),
+                                    address=cpp_zero_vclMatrix(nrow, ncol, 6L, context_index - 1),
                                     .context_index = context_index,
                                     .platform_index = platform_index,
                                     .platform = platform_name,
@@ -132,8 +123,9 @@ setMethod('vclMatrix',
                                     .device = device_name)
                             },
                             double = {
+                                assert_has_double(platform_index, device_index)
                                 new("dvclMatrix",
-                                    address = cpp_zero_vclMatrix(nrow, ncol, 8L, device_flag),
+                                    address = cpp_zero_vclMatrix(nrow, ncol, 8L, context_index - 1),
                                     .context_index = context_index,
                                     .platform_index = platform_index,
                                     .platform = platform_name,
@@ -155,10 +147,10 @@ setMethod('vclMatrix',
 #' @aliases vclMatrix,numeric
 setMethod('vclMatrix', 
           signature(data = 'numeric'),
-          function(data, nrow, ncol, type=NULL){
+          function(data, nrow, ncol, type=NULL, ctx_id=NULL){
               
               if (is.null(type)) type <- getOption("gpuR.default.type")
-              device_flag <- ifelse(options("gpuR.default.device.type") == "gpu", 0, 1)
+#               device_flag <- ifelse(options("gpuR.default.device.type") == "gpu", 0, 1)
               
               if(is.na(nrow)) stop("must indicate number of rows: nrow")
               if(is.na(ncol)) stop("must indicate number of columns: ncol")
@@ -166,9 +158,9 @@ setMethod('vclMatrix',
               assert_is_numeric(ncol)
               
               if(length(data) == 1){
-                  data <- vclMatInitNumScalar(data, nrow, ncol, type, device_flag)
+                  data <- vclMatInitNumScalar(data, nrow, ncol, type, ctx_id)
               }else{
-                  data <- vclMatInitNumVec(data, nrow, ncol, type, device_flag)
+                  data <- vclMatInitNumVec(data, nrow, ncol, type, ctx_id)
               }
               
               return(data)
@@ -179,7 +171,7 @@ setMethod('vclMatrix',
 #' @aliases vclMatrix,integer
 setMethod('vclMatrix',
           signature(data = 'integer'),
-          function(data, nrow, ncol, type=NULL){
+          function(data, nrow, ncol, type=NULL, ctx_id=NULL){
               
               if (is.null(type)) type <- "integer"
               device_flag <- ifelse(options("gpuR.default.device.type") == "gpu", 0, 1)
@@ -190,9 +182,9 @@ setMethod('vclMatrix',
               assert_is_numeric(ncol)
               
               if(length(data) == 1){
-                  data <- vclMatInitIntScalar(data, nrow, ncol, type, device_flag)
+                  data <- vclMatInitIntScalar(data, nrow, ncol, type, ctx_id)
               }else{
-                  data <- vclMatInitIntVec(data, nrow, ncol, type, device_flag)
+                  data <- vclMatInitIntVec(data, nrow, ncol, type, ctx_id)
               }
               
               return(data)
