@@ -1,9 +1,6 @@
 library(gpuR)
 context("CPU gpuMatrix algebra")
 
-# set option to use CPU instead of GPU
-options(gpuR.default.device.type = "cpu")
-
 # set seed
 set.seed(123)
 
@@ -26,12 +23,15 @@ test_that("CPU gpuMatrix Single Precision Matrix multiplication", {
     
     fgpuA <- gpuMatrix(A, type="float")
     fgpuB <- gpuMatrix(B, type="float")
+    fgpuE <- gpuMatrix(E, type = "float")
     
     fgpuC <- fgpuA %*% fgpuB
     
     expect_is(fgpuC, "fgpuMatrix")
     expect_equal(fgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent")  
+    expect_error(fgpuA %*% fgpuE,
+                 info = "error not thrown for non-conformant matrices")
 })
 
 test_that("CPU gpuMatrix Single Precision Matrix Subtraction", {
@@ -224,14 +224,18 @@ test_that("CPU gpuMatrix Single Precision Scalar Matrix Power", {
     has_cpu_skip()
     
     C <- A^2
+    C2 <- 2^A
     
     dgpuA <- gpuMatrix(A, type="float")
     
     dgpuC <- dgpuA^2
+    dgpuC2 <- 2^dgpuA
     
     expect_is(dgpuC, "fgpuMatrix")
     expect_equal(dgpuC[,], C, tolerance=1e-07, 
                  info="float matrix elements not equivalent") 
+    expect_equal(dgpuC2[,], C2, tolerance=1e-07, 
+                 info="float matrix elements not equivalent") 
 })
 
 test_that("CPU gpuMatrix Single Precision crossprod", {
@@ -313,8 +317,8 @@ test_that("CPU gpuMatrix Integer Matrix multiplication", {
     
     igpuC <- igpuA %*% igpuB
     
-    expect_equivalent(igpuC[,], Cint, 
-                      info="integer matrix elements not equivalent")      
+    expect_equivalent(igpuC[,], Cint,
+                      info="integer matrix elements not equivalent")
 })
 
 test_that("CPU gpuMatrix Integer Matrix Subtraction", {
@@ -329,8 +333,43 @@ test_that("CPU gpuMatrix Integer Matrix Subtraction", {
     igpuC <- igpuA - igpuB
     
     expect_is(igpuC, "igpuMatrix")
-    expect_equal(igpuC[,], Cint, 
-                 info="integer matrix elements not equivalent")  
+    expect_equal(igpuC[,], Cint,
+                 info="integer matrix elements not equivalent")
+})
+
+test_that("CPU gpuMatrix Integer Precision Scalar Matrix Subtraction", {
+    
+    has_cpu_skip()
+    
+    C <- Aint - 1L
+    C2 <- 1L - Aint
+    
+    fgpuA <- gpuMatrix(Aint, type="integer")
+    
+    fgpuC <- fgpuA - 1L
+    fgpuC2 <- 1L - fgpuA
+    
+    expect_is(fgpuC, "igpuMatrix")
+    expect_equal(fgpuC[,], C, 
+                 info="integer matrix elements not equivalent") 
+    expect_is(fgpuC2, "igpuMatrix")
+    expect_equal(fgpuC2[,], C2,
+                 info="intger matrix elements not equivalent") 
+})
+
+test_that("CPU gpuMatrix Integer Precision Unary Scalar Matrix Subtraction", {
+    
+    has_cpu_skip()
+    
+    C <- -Aint
+    
+    fgpuA <- gpuMatrix(Aint, type="integer")
+    
+    fgpuC <- -fgpuA
+    
+    expect_is(fgpuC, "igpuMatrix")
+    expect_equal(fgpuC[,], C,
+                 info="integer matrix elements not equivalent") 
 })
 
 test_that("CPU gpuMatrix Integer Matrix Addition", {
@@ -346,9 +385,145 @@ test_that("CPU gpuMatrix Integer Matrix Addition", {
     
     expect_is(igpuC, "igpuMatrix")
     expect_equal(igpuC[,], Cint,
+                 info="integer matrix elements not equivalent")
+})
+
+test_that("CPU gpuMatrix Integer Precision Scalar Matrix Addition", {
+    
+    has_cpu_skip()
+    
+    C <- Aint + 1L
+    C2 <- 1L + Aint
+    
+    fgpuA <- gpuMatrix(Aint, type="integer")
+    
+    fgpuC <- fgpuA + 1L
+    fgpuC2 <- 1L + fgpuA
+    
+    expect_is(fgpuC, "igpuMatrix")
+    expect_equal(fgpuC[,], C,
+                 info="integer matrix elements not equivalent") 
+    expect_is(fgpuC2, "igpuMatrix")
+    expect_equal(fgpuC2[,], C2,
+                 info="integer matrix elements not equivalent") 
+})
+
+test_that("CPU gpuMatrix Integer Precision Matrix Element-Wise Multiplication", {
+    
+    has_cpu_skip()
+    
+    C <- Aint * Bint
+    
+    fgpuA <- gpuMatrix(Aint, type="integer")
+    fgpuB <- gpuMatrix(Bint, type="integer")
+    
+    fgpuC <- fgpuA * fgpuB
+    
+    expect_is(fgpuC, "igpuMatrix")
+    expect_equal(fgpuC[,], C, 
                  info="integer matrix elements not equivalent")  
 })
 
+test_that("CPU gpuMatrix Integer Precision Scalar Matrix Multiplication", {
+    
+    has_cpu_skip()
+    
+    C <- Aint * 2L
+    C2 <- 2L * Aint
+    
+    dgpuA <- gpuMatrix(Aint, type="integer")
+    
+    dgpuC <- dgpuA * 2L
+    dgpuC2 <- 2L * dgpuA
+    
+    expect_is(dgpuC, "igpuMatrix")
+    expect_equal(dgpuC[,], C,
+                 info="integer matrix elements not equivalent") 
+    expect_is(dgpuC2, "igpuMatrix")
+    expect_equal(dgpuC2[,], C2,
+                 info="integer matrix elements not equivalent") 
+})
+
+test_that("CPU gpuMatrix Integer Precision Matrix Element-Wise Division", {
+    
+    has_cpu_skip()
+    
+    C <- Aint / Bint
+    C <- apply(C, 2, as.integer)
+    
+    fgpuA <- gpuMatrix(Aint, type="integer")
+    fgpuB <- gpuMatrix(Bint, type="integer")
+    
+    fgpuC <- fgpuA / fgpuB
+    
+    expect_is(fgpuC, "igpuMatrix")
+    expect_equal(fgpuC[,], C,
+                 info="integer matrix elements not equivalent")  
+})
+
+test_that("CPU gpuMatrix Integer Precision Scalar Matrix Division", {
+    
+    has_cpu_skip()
+    
+    C <- Aint/2L
+    C2 <- 2L/Aint
+    
+    C <- apply(C, 2, as.integer)
+    C2 <- apply(C2, 2, as.integer)
+    
+    dgpuA <- gpuMatrix(Aint, type="integer")
+    
+    dgpuC <- dgpuA/2L
+    dgpuC2 <- 2L/dgpuA
+    
+    expect_is(dgpuC, "igpuMatrix")
+    expect_equal(dgpuC[,], C,
+                 info="integer matrix elements not equivalent") 
+    expect_is(dgpuC2, "igpuMatrix")
+    expect_equal(dgpuC2[,], C2,
+                 info="integer matrix elements not equivalent") 
+})
+
+test_that("CPU gpuMatrix Integer Precision Matrix Element-Wise Power", {
+    
+    has_cpu_skip()
+    
+    Apow <- matrix(seq.int(9), ncol=3, nrow=3)
+    Bpow <- matrix(2, ncol = 3, nrow = 3)
+    C <- Apow ^ Bpow
+    
+    fgpuA <- gpuMatrix(Apow, type="integer")
+    fgpuB <- gpuMatrix(Bpow, type="integer")
+    
+    fgpuC <- fgpuA ^ fgpuB
+    
+    expect_is(fgpuC, "igpuMatrix")
+    expect_equal(fgpuC[,], C,
+                 info="integer matrix elements not equivalent")
+})
+
+test_that("CPU gpuMatrix Integer Precision Scalar Matrix Power", {
+    
+    has_cpu_skip()
+    
+    C <- Aint^2L
+    C2 <- 2L^Aint
+    
+    C <- apply(C, 2, as.integer)
+    C2 <- apply(C2, 2, as.integer)
+    
+    dgpuA <- gpuMatrix(Aint, type="integer")
+    
+    dgpuC <- dgpuA^2L
+    dgpuC2 <- 2L^dgpuA
+    
+    expect_is(dgpuC, "igpuMatrix")
+    expect_equal(dgpuC[,], C,
+                 info="integer matrix elements not equivalent")
+    expect_equal(dgpuC2[,], C2,
+                 info="integer matrix elements not equivalent")
+})
+
 # Double Precision tests
 
 test_that("CPU gpuMatrix Double Precision Matrix multiplication", {
@@ -646,5 +821,40 @@ test_that("CPU gpuMatrix Double Precision transpose", {
 })
 
 
-# set option back to GPU
-options(gpuR.default.device.type = "gpu")
+test_that("CPU gpuMatrix Diagonal access", {
+    
+    has_cpu_skip()
+    
+    fgpuA <- gpuMatrix(A, type="float")
+    
+    D <- diag(A)
+    gpuD <- diag(fgpuA)
+        
+    expect_is(gpuD, "fgpuVector")
+    expect_equal(gpuD[,], D, tolerance=1e-07, 
+                 info="float matrix diagonal elements not equivalent") 
+    
+    vec <- rnorm(ORDER)
+    diag(fgpuA) <- gpuVector(vec, type = "float")
+    diag(A) <- vec
+    
+    expect_equal(fgpuA[,], A, tolerance=1e-07, 
+                 info="set float matrix diagonal elements not equivalent") 
+    
+    
+    fgpuA <- gpuMatrix(A, type="double")
+    
+    D <- diag(A)
+    gpuD <- diag(fgpuA)
+    
+    expect_is(gpuD, "dgpuVector")
+    expect_equal(gpuD[,], D, tolerance=.Machine$double.eps^0.5, 
+                 info="double matrix diagonal elements not equivalent")  
+    
+    vec <- rnorm(ORDER)
+    diag(fgpuA) <- gpuVector(vec, type = "double")
+    diag(A) <- vec
+    
+    expect_equal(fgpuA[,], A, tolerance=.Machine$double.eps^0.5, 
+                 info="set double matrix diagonal elements not equivalent") 
+})
