@@ -1,8 +1,5 @@
 library(gpuR)
 context("CPU gpuMatrix math operations")
-
-# set option to use CPU instead of GPU
-options(gpuR.default.device = "cpu")
 options(warn=-1)
 
 # set seed
@@ -20,10 +17,10 @@ test_that("CPU gpuMatrix Single Precision Matrix Element-Wise Trignometry", {
     has_cpu_skip()
     
     Sin <- sin(A)
-    Asin <- asin(A)
+    Asin <- suppressWarnings(asin(A))
     Hsin <- sinh(A)
     Cos <- cos(A)
-    Acos <- acos(A)
+    Acos <- suppressWarnings(acos(A))
     Hcos <- cosh(A)
     Tan <- tan(A) 
     Atan <- atan(A)
@@ -75,10 +72,10 @@ test_that("CPU gpuMatrix Double Precision Matrix Element-Wise Trignometry", {
     has_cpu_skip()
     
     Sin <- sin(A)
-    Asin <- asin(A)
+    Asin <- suppressWarnings(asin(A))
     Hsin <- sinh(A)
     Cos <- cos(A)
-    Acos <- acos(A)
+    Acos <- suppressWarnings(acos(A))
     Hcos <- cosh(A)
     Tan <- tan(A) 
     Atan <- atan(A)
@@ -129,9 +126,9 @@ test_that("CPU gpuMatrix Double Precision Matrix Element-Wise Trignometry", {
 test_that("CPU gpuMatrix Single Precision Matrix Element-Wise Logs", {
     has_cpu_skip()
     
-    R_log <- log(A)
-    R_log10 <- log10(A)
-    R_log2 <- log(A, base=2)
+    R_log <- suppressWarnings(log(A))
+    R_log10 <- suppressWarnings(log10(A))
+    R_log2 <- suppressWarnings(log(A, base=2))
     
     fgpuA <- gpuMatrix(A, type="float")
     
@@ -156,9 +153,9 @@ test_that("CPU gpuMatrix Single Precision Matrix Element-Wise Logs", {
 test_that("CPU gpuMatrix Double Precision Matrix Element-Wise Logs", {
     has_cpu_skip()
     
-    R_log <- log(A)
-    R_log10 <- log10(A)
-    R_log2 <- log(A, base=2)
+    R_log <- suppressWarnings(log(A))
+    R_log10 <- suppressWarnings(log10(A))
+    R_log2 <- suppressWarnings(log(A, base=2))
     
     fgpuA <- gpuMatrix(A, type="double")
     
@@ -180,7 +177,7 @@ test_that("CPU gpuMatrix Double Precision Matrix Element-Wise Logs", {
                  check.attributes=FALSE) 
 })
 
-test_that("gpuMatrix Single Precision Matrix Exponential", {
+test_that("CPU gpuMatrix Single Precision Matrix Exponential", {
     has_cpu_skip()
     
     R_exp <- exp(A)
@@ -195,7 +192,7 @@ test_that("gpuMatrix Single Precision Matrix Exponential", {
                  check.attributes=FALSE)  
 })
 
-test_that("gpuMatrix Double Precision Matrix Exponential", {
+test_that("CPU gpuMatrix Double Precision Matrix Exponential", {
     has_cpu_skip()
     
     R_exp <- exp(A)
@@ -205,10 +202,78 @@ test_that("gpuMatrix Double Precision Matrix Exponential", {
     fgpu_exp <- exp(fgpuA)
     
     expect_is(fgpu_exp, "dgpuMatrix")
-    expect_equal(fgpu_exp[,], R_exp, tolerance=1e-07, 
+    expect_equal(fgpu_exp[,], R_exp, tolerance=.Machine$double.eps^0.5, 
                  info="exp double matrix elements not equivalent",
                  check.attributes=FALSE)  
 })
 
-options(gpuR.default.device = "gpu")
+test_that("CPU gpuMatrix Single Precision Matrix Absolute Value", {
+    
+    has_cpu_skip()
+    
+    R_abs <- abs(A)
+    
+    fgpuA <- gpuMatrix(A, type="float")
+    
+    fgpu_abs <- abs(fgpuA)
+    
+    expect_is(fgpu_abs, "fgpuMatrix")
+    expect_equal(fgpu_abs[,], R_abs, tolerance=1e-07, 
+                 info="abs float matrix elements not equivalent")  
+})
+
+test_that("CPU gpuMatrix Double Precision Matrix Absolute Value", {
+    
+    has_cpu_skip()
+    
+    R_abs <- abs(A)
+    
+    fgpuA <- gpuMatrix(A, type="double")
+    
+    fgpu_abs <- abs(fgpuA)
+    
+    expect_is(fgpu_abs, "dgpuMatrix")
+    expect_equal(fgpu_abs[,], R_abs, tolerance=.Machine$double.eps^0.5, 
+                 info="abs double matrix elements not equivalent")  
+})
+
+
+test_that("gpuMatrix Single Precision Maximum/Minimum", {
+    
+    has_cpu_skip()
+    
+    R_max <- max(A)
+    R_min <- min(A)
+    
+    fgpuA <- gpuMatrix(A, type="float")
+    
+    fgpu_max <- max(fgpuA)
+    fgpu_min <- min(fgpuA)
+    
+    expect_is(fgpu_max, "numeric")
+    expect_equal(fgpu_max, R_max, tolerance=1e-07, 
+                 info="max float matrix element not equivalent")  
+    expect_equal(fgpu_min, R_min, tolerance=1e-07, 
+                 info="min float matrix element not equivalent")  
+})
+
+test_that("gpuMatrix Double Precision Maximum/Minimum", {
+    
+    has_cpu_skip()
+    
+    R_max <- max(A)
+    R_min <- min(A)
+    
+    fgpuA <- gpuMatrix(A, type="double")
+    
+    fgpu_max <- max(fgpuA)
+    fgpu_min <- min(fgpuA)
+    
+    expect_is(fgpu_max, "numeric")
+    expect_equal(fgpu_max, R_max, tolerance=.Machine$double.eps^0.5, 
+                 info="max double matrix element not equivalent") 
+    expect_equal(fgpu_min, R_min, tolerance=.Machine$double.eps^0.5, 
+                 info="min double matrix element not equivalent")  
+})
+
 options(warn=0)
