@@ -1,15 +1,13 @@
 library(gpuR)
 context("CPU gpuMatrix Utility Functions")
 
-# set option to use CPU instead of GPU
-options(gpuR.default.device = "cpu")
-
 set.seed(123)
 A <- matrix(sample(seq.int(100), 100), 10)
 D <- matrix(sample(rnorm(100), 100), 10)
 
+cnames <- paste0("V", seq(10))
 
-test_that("gpuMatrix element access", {
+test_that("CPU gpuMatrix element access", {
     has_cpu_skip()
     
     dgpu <- gpuMatrix(D)
@@ -36,9 +34,18 @@ test_that("gpuMatrix element access", {
                  info = "float element subset not equivalent ")
     expect_equivalent(igpu[1,2], A[1,2],
                       info = "integer element subset not equivalent")
+    expect_equivalent(dgpu[c(3,5)], D[c(3,5)],
+                      info = "double non-contiguous subset not equivalent")
+    
+    expect_equivalent(igpu[1:4,1:4], A[1:4,1:4],
+                      info = "row & column subsets of igpuMatrix not equivalent")
+    expect_equal(fgpu[1:4,1:4], D[1:4,1:4], tolerance = 1e-07,
+                 info = "row & column subsets of fgpuMatrix not equivalent")
+    expect_equivalent(dgpu[1:4,1:4], D[1:4,1:4],
+                      info = "row & column subsets of dgpuMatrix not equivalent")
 })
 
-test_that("gpuMatrix set column access", {
+test_that("CPU gpuMatrix set column access", {
     has_cpu_skip()
     
     gpuA <- gpuMatrix(A)
@@ -74,9 +81,16 @@ test_that("gpuMatrix set column access", {
                  info = "no error when index greater than dims")
     expect_error(gpuD[,1] <- rnorm(12),
                  info = "no error when vector larger than number of rows")
+    
+    expect_equivalent(gpuA[,1:4], A[,1:4],
+                      info = "column subsets of igpuMatrix not equivalent")
+    expect_equivalent(gpuD[,1:4], D[,1:4],
+                      info = "column subsets of fgpuMatrix not equivalent")
+    expect_equal(gpuF[,1:4], D[,1:4], tolerance = 1e-07,
+                 info = "column subsets of dgpuMatrix not equivalent")
 })
 
-test_that("gpuMatrix set row access", {
+test_that("CPU gpuMatrix set row access", {
     has_cpu_skip()
     
     gpuA <- gpuMatrix(A)
@@ -112,9 +126,16 @@ test_that("gpuMatrix set row access", {
                  info = "no error when index greater than dims")
     expect_error(gpuD[1,] <- rnorm(12),
                  info = "no error when vector larger than number of rows")
+    
+    expect_equivalent(gpuA[1:4,], A[1:4,],
+                      info = "row subsets of igpuMatrix not equivalent")
+    expect_equivalent(gpuD[1:4,], D[1:4,], 
+                      info = "row subsets of fgpuMatrix not equivalent")
+    expect_equal(gpuF[1:4,], D[1:4,], tolerance = 1e-07,
+                 info = "row subsets of dgpuMatrix not equivalent")
 })
 
-test_that("gpuMatrix set element access", {
+test_that("CPU gpuMatrix set element access", {
     has_cpu_skip()
     
     gpuA <- gpuMatrix(A)
@@ -132,6 +153,10 @@ test_that("gpuMatrix set element access", {
     A[1,3] <- int
     D[1,3] <- float
     
+    D[c(6,10)] <- 0
+    gpuD[c(6,10)] <- 0
+    gpuF[c(6,10)] <- 0
+    
     expect_equivalent(gpuD[1,3], float,
                       info = "updated dgpuMatrix element not equivalent")
     expect_equivalent(gpuD[], D,
@@ -150,6 +175,73 @@ test_that("gpuMatrix set element access", {
                  info = "no error when index greater than dims")
     expect_error(gpuD[1,3] <- rnorm(12),
                  info = "no error when assigned vector to element")
+    expect_equivalent(gpuD[c(6,10)], D[c(6,10)],
+                      info = "double non-contiguous subset not equivalent")
 })
 
-options(gpuR.default.device = "gpu")
+test_that("CPU gpuMatrix confirm print doesn't error", {
+    
+    has_cpu_skip()
+    
+    dgpu <- gpuMatrix(D, type="float")
+    
+    expect_that(print(dgpu), prints_text("Source: gpuR Matrix"))
+})
+
+test_that("CPU gpuMatrix as.matrix method", {
+    
+    has_cpu_skip()
+    
+    dgpu <- gpuMatrix(D)
+    fgpu <- gpuMatrix(D, type="float")
+    igpu <- gpuMatrix(A)
+    
+    expect_equal(as.matrix(dgpu), D,
+                      info = "double as.matrix not equal")
+    expect_equal(as.matrix(fgpu), D,
+                      info = "float as.matrix not equal",
+                      tolerance = 1e-07)
+    expect_equal(as.matrix(dgpu), D,
+                      info = "integer as.matrix not equal")
+    
+    
+    expect_is(as.matrix(dgpu), 'matrix',
+              info = "double as.matrix not producing 'matrix' class")
+    expect_is(as.matrix(fgpu), 'matrix',
+              info = "float as.matrix not producing 'matrix' class")
+    expect_is(as.matrix(igpu), 'matrix',
+              info = "integer as.matrix not producing 'matrix' class")
+})
+
+test_that("CPU gpuMatrix colnames methods", {
+    
+    has_cpu_skip()
+    
+    fgpu <- gpuMatrix(D, type="float")
+    igpu <- gpuMatrix(A)
+    
+    expect_null(colnames(fgpu), 
+                info = "float colnames should return NULL before assignment")
+    expect_null(colnames(igpu), 
+                info = "integer colnames should return NULL before assignment")
+    
+    colnames(fgpu) <- cnames
+    colnames(igpu) <- cnames
+    
+    expect_equal(colnames(fgpu), cnames,
+                 info = "float colnames don't reflect assigned names")
+    expect_equal(colnames(igpu), cnames,
+                 info = "integer colnames don't reflect assigned names")
+    
+    # Double tests
+    
+    dgpu <- gpuMatrix(D)
+    
+    expect_null(colnames(dgpu), 
+                info = "double colnames should return NULL before assignment")
+    
+    colnames(dgpu) <- cnames
+    
+    expect_equal(colnames(dgpu), cnames,
+                 info = "double colnames don't reflect assigned names")
+})
