@@ -1,9 +1,6 @@
 library(gpuR)
 context("CPU gpuVector Utility Functions")
 
-# set option to use CPU instead of GPU
-options(gpuR.default.device = "cpu")
-
 set.seed(123)
 ORDER <- 100
 A <- sample(seq.int(10), ORDER, replace = TRUE)
@@ -109,5 +106,28 @@ test_that("dgpuVector set accession method successful", {
                  info = "no error when set outside dgpuVector size")
 })
 
-options(gpuR.default.device = "gpu")
+test_that("gpuVector as.vector method", {
+    
+    has_cpu_skip()
+    
+    dgpu <- gpuVector(D)
+    fgpu <- gpuVector(D, type="float")
+    igpu <- gpuVector(A)
+    
+    expect_equal(as.vector(dgpu), D,
+                      info = "double as.vector not equal")
+    expect_equal(as.vector(fgpu), D,
+                      info = "float as.vector not equal",
+                      tolerance = 1e-07)
+    expect_equal(as.vector(dgpu), D,
+                      info = "integer as.vector not equal")
+    
+    
+    expect_is(as.vector(dgpu), 'numeric',
+              info = "double as.vector not producing 'vector' class")
+    expect_is(as.vector(fgpu), 'numeric',
+              info = "float as.vector not producing 'vector' class")
+    expect_is(as.vector(igpu), 'integer',
+              info = "integer as.vector not producing 'vector' class")
+})
 
