@@ -7,9 +7,6 @@ set.seed(123)
 # ignore warnings (logs and arc trigs)
 options(warn=-1)
 
-# set option to use CPU instead of GPU
-options(gpuR.default.device = "cpu")
-
 ORDER <- 4
 
 # Base R objects
@@ -18,14 +15,14 @@ B <- matrix(rnorm(ORDER^2), nrow=ORDER, ncol=ORDER)
 E <- matrix(rnorm(15), nrow=5)
 
 
-test_that("vclMatrix Single Precision Matrix Element-Wise Trignometry", {
+test_that("CPU vclMatrix Single Precision Matrix Element-Wise Trignometry", {
     has_cpu_skip()
     
     Sin <- sin(A)
-    Asin <- asin(A)
+    Asin <- suppressWarnings(asin(A))
     Hsin <- sinh(A)
     Cos <- cos(A)
-    Acos <- acos(A)
+    Acos <- suppressWarnings(acos(A))
     Hcos <- cosh(A)
     Tan <- tan(A) 
     Atan <- atan(A)
@@ -64,14 +61,14 @@ test_that("vclMatrix Single Precision Matrix Element-Wise Trignometry", {
                  info="hyperbolic tan float matrix elements not equivalent")  
 })
 
-test_that("vclMatrix Double Precision Matrix Element-Wise Trignometry", {
+test_that("CPU vclMatrix Double Precision Matrix Element-Wise Trignometry", {
     has_cpu_skip()
     
     Sin <- sin(A)
-    Asin <- asin(A)
+    Asin <- suppressWarnings(asin(A))
     Hsin <- sinh(A)
     Cos <- cos(A)
-    Acos <- acos(A)
+    Acos <- suppressWarnings(acos(A))
     Hcos <- cosh(A)
     Tan <- tan(A) 
     Atan <- atan(A)
@@ -111,12 +108,12 @@ test_that("vclMatrix Double Precision Matrix Element-Wise Trignometry", {
 })
 
 
-test_that("vclMatrix Single Precision Matrix Element-Wise Logs", {
+test_that("CPU vclMatrix Single Precision Matrix Element-Wise Logs", {
     has_cpu_skip()
     
-    R_log <- log(A)
-    R_log10 <- log10(A)
-    R_log2 <- log(A, base=2)
+    R_log <- suppressWarnings(log(A))
+    R_log10 <- suppressWarnings(log10(A))
+    R_log2 <- suppressWarnings(log(A, base=2))
     
     fgpuA <- vclMatrix(A, type="float")
     
@@ -135,12 +132,12 @@ test_that("vclMatrix Single Precision Matrix Element-Wise Logs", {
                  info="base log float matrix elements not equivalent") 
 })
 
-test_that("vclMatrix Double Precision Matrix Element-Wise Logs", {
+test_that("CPU vclMatrix Double Precision Matrix Element-Wise Logs", {
     has_cpu_skip()
     
-    R_log <- log(A)
-    R_log10 <- log10(A)
-    R_log2 <- log(A, base=2)
+    R_log <- suppressWarnings(log(A))
+    R_log10 <- suppressWarnings(log10(A))
+    R_log2 <- suppressWarnings(log(A, base=2))
     
     fgpuA <- vclMatrix(A, type="double")
     
@@ -160,7 +157,7 @@ test_that("vclMatrix Double Precision Matrix Element-Wise Logs", {
 })
 
 
-test_that("vclMatrix Single Precision Matrix Exponential", {
+test_that("CPU vclMatrix Single Precision Matrix Exponential", {
     has_cpu_skip()
     
     R_exp <- exp(A)
@@ -174,7 +171,7 @@ test_that("vclMatrix Single Precision Matrix Exponential", {
                  info="exp float matrix elements not equivalent")  
 })
 
-test_that("vclMatrix Double Precision Matrix Exponential", {
+test_that("CPU vclMatrix Double Precision Matrix Exponential", {
     has_cpu_skip()
     
     R_exp <- exp(A)
@@ -188,5 +185,72 @@ test_that("vclMatrix Double Precision Matrix Exponential", {
                  info="exp double matrix elements not equivalent")  
 })
 
-# set option back to GPU
-options(gpuR.default.device = "gpu")
+
+test_that("CPU vclMatrix Single Precision Matrix Absolute Value", {
+    
+    has_cpu_skip()
+    
+    R_abs <- abs(A)
+    
+    fvclA <- vclMatrix(A, type="float")
+    
+    fvcl_abs <- abs(fvclA)
+    
+    expect_is(fvcl_abs, "fvclMatrix")
+    expect_equal(fvcl_abs[,], R_abs, tolerance=1e-07, 
+                 info="abs float matrix elements not equivalent")  
+})
+
+test_that("CPU vclMatrix Double Precision Matrix Absolute Value", {
+    
+    has_cpu_skip()
+    
+    R_abs <- abs(A)
+    
+    fvclA <- vclMatrix(A, type="double")
+    
+    fvcl_abs <- abs(fvclA)
+    
+    expect_is(fvcl_abs, "dvclMatrix")
+    expect_equal(fvcl_abs[,], R_abs, tolerance=.Machine$double.eps^0.5, 
+                 info="abs double matrix elements not equivalent")  
+})
+
+
+test_that("CPU vclMatrix Single Precision Maximum/Minimum", {
+    
+    has_cpu_skip()
+    
+    R_max <- max(A)
+    R_min <- min(A)
+    
+    fvclA <- vclMatrix(A, type="float")
+    
+    fvcl_max <- max(fvclA)
+    fvcl_min <- min(fvclA)
+    
+    expect_is(fvcl_max, "numeric")
+    expect_equal(fvcl_max, R_max, tolerance=1e-07, 
+                 info="max float matrix element not equivalent")  
+    expect_equal(fvcl_min, R_min, tolerance=1e-07, 
+                 info="min float matrix element not equivalent")  
+})
+
+test_that("CPU vclMatrix Double Precision Maximum/Minimum", {
+    
+    has_cpu_skip()
+    
+    R_max <- max(A)
+    R_min <- min(A)
+    
+    fvclA <- vclMatrix(A, type="double")
+    
+    fvcl_max <- max(fvclA)
+    fvcl_min <- min(fvclA)
+    
+    expect_is(fvcl_max, "numeric")
+    expect_equal(fvcl_max, R_max, tolerance=.Machine$double.eps^0.5, 
+                 info="max double matrix element not equivalent") 
+    expect_equal(fvcl_min, R_min, tolerance=.Machine$double.eps^0.5, 
+                 info="min double matrix element not equivalent")  
+})
